package com.sun.jmx.snmp.IPAcl;

import java.security.acl.Permission;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.File;
import java.util.HashSet;
import java.security.acl.AclEntry;
import java.util.Enumeration;
import java.security.acl.NotOwnerException;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import java.net.UnknownHostException;
import java.util.Vector;
import java.net.InetAddress;
import java.util.Hashtable;
import java.io.Serializable;
import com.sun.jmx.snmp.InetAddressAcl;

public class SnmpAcl implements InetAddressAcl, Serializable
{
    private static final long serialVersionUID = -6702287103824397063L;
    static final PermissionImpl READ;
    static final PermissionImpl WRITE;
    private AclImpl acl;
    private boolean alwaysAuthorized;
    private String authorizedListFile;
    private Hashtable<InetAddress, Vector<String>> trapDestList;
    private Hashtable<InetAddress, Vector<String>> informDestList;
    private PrincipalImpl owner;
    
    public SnmpAcl(String s) throws UnknownHostException, IllegalArgumentException {
        this(s, null);
    }
    
    public SnmpAcl(String s, String authorizedListFile) throws UnknownHostException, IllegalArgumentException {
        this.acl = null;
        this.alwaysAuthorized = false;
        this.authorizedListFile = null;
        this.trapDestList = null;
        this.informDestList = null;
        this.owner = null;
        this.trapDestList = new Hashtable();
        this.informDestList = new Hashtable();
        this.owner = new PrincipalImpl();
        try {
            this.acl = new AclImpl(this.owner, s);
            AclEntryImpl aclEntryImpl = new AclEntryImpl(this.owner);
            aclEntryImpl.addPermission(SnmpAcl.READ);
            aclEntryImpl.addPermission(SnmpAcl.WRITE);
            this.acl.addEntry(this.owner, aclEntryImpl);
        }
        catch (NotOwnerException ex) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpAcl.class.getName(), "SnmpAcl(String,String)", "Should never get NotOwnerException as the owner is built in this constructor");
            }
        }
        if (authorizedListFile == null) {
            this.setDefaultFileName();
        }
        else {
            this.setAuthorizedListFile(authorizedListFile);
        }
        this.readAuthorizedListFile();
    }
    
    public Enumeration<AclEntry> entries() {
        return this.acl.entries();
    }
    
    public Enumeration<String> communities() {
        HashSet<Object> set = new HashSet();
        Vector<Object> vector = new Vector();
        Enumeration<AclEntry> entries = this.acl.entries();
        while (entries.hasMoreElements()) {
            Enumeration<String> communities = ((AclEntryImpl)entries.nextElement()).communities();
            while (communities.hasMoreElements()) {
                set.add(communities.nextElement());
            }
        }
        String[] array = (String[])set.toArray(new String[0]);
        for (int i = 0; i < array.length; ++i) {
            vector.addElement(array[i]);
        }
        return vector.elements();
    }
    
    public String getName() {
        return this.acl.getName();
    }
    
    public static PermissionImpl getREAD() {
        return SnmpAcl.READ;
    }
    
    public static PermissionImpl getWRITE() {
        return SnmpAcl.WRITE;
    }
    
    public static String getDefaultAclFileName() {
        String property = System.getProperty("file.separator");
        return new StringBuffer(System.getProperty("java.home")).append(property).append("lib").append(property).append("snmp.acl").toString();
    }
    
    public void setAuthorizedListFile(String s) throws IllegalArgumentException {
        File obj = new File(s);
        if (!obj.isFile()) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpAcl.class.getName(), "setAuthorizedListFile", "ACL file not found: " + s);
            }
            throw new IllegalArgumentException("The specified file [" + obj + "] doesn't exist or is not a file, no configuration loaded");
        }
        else {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "setAuthorizedListFile", "Default file set to " + s);
            }
            this.authorizedListFile = s;
            return;
        }
    }
    
    public void rereadTheFile() throws NotOwnerException, UnknownHostException {
        this.alwaysAuthorized = false;
        this.acl.removeAll(this.owner);
        this.trapDestList.clear();
        this.informDestList.clear();
        AclEntryImpl aclEntryImpl = new AclEntryImpl(this.owner);
        aclEntryImpl.addPermission(SnmpAcl.READ);
        aclEntryImpl.addPermission(SnmpAcl.WRITE);
        this.acl.addEntry(this.owner, aclEntryImpl);
        this.readAuthorizedListFile();
    }
    
    public String getAuthorizedListFile() {
        return this.authorizedListFile;
    }
    
    public boolean checkReadPermission(InetAddress inetAddress) {
        return this.alwaysAuthorized || this.acl.checkPermission(new PrincipalImpl(inetAddress), SnmpAcl.READ);
    }
    
    public boolean checkReadPermission(InetAddress inetAddress, String s) {
        return this.alwaysAuthorized || this.acl.checkPermission(new PrincipalImpl(inetAddress), s, SnmpAcl.READ);
    }
    
    public boolean checkCommunity(String s) {
        return this.acl.checkCommunity(s);
    }
    
    public boolean checkWritePermission(InetAddress inetAddress) {
        return this.alwaysAuthorized || this.acl.checkPermission(new PrincipalImpl(inetAddress), SnmpAcl.WRITE);
    }
    
    public boolean checkWritePermission(InetAddress inetAddress, String s) {
        return this.alwaysAuthorized || this.acl.checkPermission(new PrincipalImpl(inetAddress), s, SnmpAcl.WRITE);
    }
    
    public Enumeration<InetAddress> getTrapDestinations() {
        return this.trapDestList.keys();
    }
    
    public Enumeration<String> getTrapCommunities(InetAddress key) {
        Vector<Object> vector;
        if ((vector = (Vector)this.trapDestList.get(key)) != null) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "getTrapCommunities", "[" + key.toString() + "] is in list");
            }
            return vector.elements();
        }
        else {
            Vector<Object> vector2 = new Vector();
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "getTrapCommunities", "[" + key.toString() + "] is not in list");
            }
            return vector2.elements();
        }
    }
    
    public Enumeration<InetAddress> getInformDestinations() {
        return this.informDestList.keys();
    }
    
    public Enumeration<String> getInformCommunities(InetAddress key) {
        Vector<Object> vector;
        if ((vector = (Vector)this.informDestList.get(key)) != null) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "getInformCommunities", "[" + key.toString() + "] is in list");
            }
            return vector.elements();
        }
        else {
            Vector<Object> vector2 = new Vector();
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "getInformCommunities", "[" + key.toString() + "] is not in list");
            }
            return vector2.elements();
        }
    }
    
    private void readAuthorizedListFile() {
        this.alwaysAuthorized = false;
        if (this.authorizedListFile == null) {
            if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "readAuthorizedListFile", "alwaysAuthorized set to true");
            }
            this.alwaysAuthorized = true;
        }
        else {
            Parser parser;
            try {
                parser = new Parser(new FileInputStream(this.getAuthorizedListFile()));
            }
            catch (FileNotFoundException ex) {
                if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpAcl.class.getName(), "readAuthorizedListFile", "The specified file was not found, authorize everybody");
                }
                this.alwaysAuthorized = true;
                return;
            }
            try {
                JDMSecurityDefs securityDefs = parser.SecurityDefs();
                securityDefs.buildAclEntries(this.owner, this.acl);
                securityDefs.buildTrapEntries(this.trapDestList);
                securityDefs.buildInformEntries(this.informDestList);
            }
            catch (ParseException thrown) {
                if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpAcl.class.getName(), "readAuthorizedListFile", "Got parsing exception", thrown);
                }
                throw new IllegalArgumentException(thrown.getMessage());
            }
            catch (Error thrown2) {
                if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINEST)) {
                    JmxProperties.SNMP_LOGGER.logp(Level.FINEST, SnmpAcl.class.getName(), "readAuthorizedListFile", "Got unexpected error", thrown2);
                }
                throw new IllegalArgumentException(thrown2.getMessage());
            }
            Enumeration<AclEntry> entries = this.acl.entries();
            while (entries.hasMoreElements()) {
                AclEntryImpl aclEntryImpl = (AclEntryImpl)entries.nextElement();
                if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                    JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "readAuthorizedListFile", "===> " + aclEntryImpl.getPrincipal().toString());
                }
                Enumeration<Permission> permissions = aclEntryImpl.permissions();
                while (permissions.hasMoreElements()) {
                    Permission obj = (Permission)permissions.nextElement();
                    if (JmxProperties.SNMP_LOGGER.isLoggable(Level.FINER)) {
                        JmxProperties.SNMP_LOGGER.logp(Level.FINER, SnmpAcl.class.getName(), "readAuthorizedListFile", "perm = " + obj);
                    }
                    else {
                        continue;
                    }
                }
            }
        }
    }
    
    private void setDefaultFileName() {
        try {
            this.setAuthorizedListFile(getDefaultAclFileName());
        }
        catch (IllegalArgumentException ex) {}
    }
    
    static {
        READ = new PermissionImpl("READ");
        WRITE = new PermissionImpl("WRITE");
    }
}
